//******************************************************************************
//  Description:    Communication with the LMP91000 via I2C bus
//
//  Demo Application for MSP430/LMP91000 Interface Code Library v1.0
//  Read/Write LMP91000 Registers
//  Works on LMP91002 as well
//
//                MSP430F5528
//             -----------------
//         /|\|              XIN|-
//          | |                 |  
//          --|RST          XOUT|-
//            |                 |
//            |     P3.0/UCB0SDA|<--> SDA 
//            |     P3.1/UCB0SCL|--> SCL 
//            |                 | 
//            |                 | 
//            |             P2.6|<-- MENB 
//            |                 | 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   July 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
// 
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.10
// Comments: Modified register read/write values to support LMP91002 as well
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************

/*  Copyright 2007-2009 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller or used
  solely and exclusively in conjunction with a Texas Instruments radio frequency
  transceiver, which is integrated into your product. Other than for the
  foregoing purpose, you may not use, reproduce, copy, prepare derivative works
  of, modify, distribute, perform, display or sell this Software and/or its
  documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91000.h"
#include "TI_LMP91000_register_settings.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_i2c.h"
/*----------------------------------------------------------------------------*/
#define TIACN_NEW_VALUE 0x18                                                   // Test value for TIACN Register Write
/*----------------------------------------------------------------------------*/
void main(void)
{

  uint8_t read_val[4];
  uint8_t status = TI_LMP91000_NOT_READY;  

  WDTCTL = WDTPW + WDTHOLD;                                                    // Stop watchdog timer
  
  TI_LMP91000_LED_PxOUT |= TI_LMP91000_LED_PIN;                                // Set LED ON
  TI_LMP91000_LED_PxDIR |= TI_LMP91000_LED_PIN;                                // Set pin direction is output  

  I2CSetup(LMP91000_I2C_Address);                                              // Initialize I2C module
  
  TI_LMP91000_MENB_PxOUT &= ~TI_LMP91000_MENB_PIN;                             // Enable \MENB Pin
  TI_LMP91000_MENB_PxDIR |= TI_LMP91000_MENB_PIN;                              // Set pin direction is output
 
  while (status == TI_LMP91000_NOT_READY)                                      // wait while device is not ready
    status = LMP91000_I2CReadReg(TI_LMP91000_STATUS_REG);                       // Read device ready status
      
  read_val[0] = LMP91000_I2CReadReg(TI_LMP91000_LOCK_REG);                     // Read from lock register default value
  read_val[1] = LMP91000_I2CReadReg(TI_LMP91000_TIACN_REG);                    // Read TIA control register default value
  read_val[2] = LMP91000_I2CReadReg(TI_LMP91000_REFCN_REG);                    // Read Reference control register default value
  read_val[3] = LMP91000_I2CReadReg(TI_LMP91000_MODECN_REG);                   // Read Mode control register default value
  
  LMP91000_I2CWriteReg(TI_LMP91000_LOCK_REG, TI_LMP91000_WRITE_UNLOCK);        // unlock the registers for write
 
  LMP91000_I2CWriteReg(TI_LMP91000_TIACN_REG, TIACN_NEW_VALUE);                // Modify TIA control register
  read_val[1] = LMP91000_I2CReadReg(TI_LMP91000_TIACN_REG);                    // Read to confirm register is modified
  
  // test if write/read values match
  if (read_val[1] == TIACN_NEW_VALUE)
  {
    while (1)                                                                  // no error: blink LED continuously
    {
        __delay_cycles(250000);
        __delay_cycles(250000);
        TI_LMP91000_LED_PxOUT ^= TI_LMP91000_LED_PIN;        
    } 
  } else
  {
    TI_LMP91000_LED_PxOUT &= ~TI_LMP91000_LED_PIN;                             // error: Set LED OFF   
  }   
  
  LMP91000_I2CWriteReg(TI_LMP91000_TIACN_REG, TI_LMP91000_TIACN_REG_VALUE);    // store default value back in TIACN  
  LMP91000_I2CWriteReg(TI_LMP91000_LOCK_REG, TI_LMP91000_WRITE_LOCK);          // lock the registers  
  TI_LMP91000_MENB_PxOUT |= TI_LMP91000_MENB_PIN;                              // Disable \MENB Pin   
  
  __bis_SR_register(LPM0_bits);                                                // Enter LPM0, Enable interrupts
  __no_operation();                                                            // For Debugger

}
/*----------------------------------------------------------------------------*/