//******************************************************************************
//  Description: Temperature Measurement using LMP91000 & MSP430F5528
//
//  Demo Application for MSP430/LMP91000 Interface Code Library v1.0
//  This demo application is not applicable to the LMP91002 devices as they don't 
//  support an on-chip temperature sensor.
//
//  This example shows how to perform Temperature Measurement using LMP91000
//  and MSP430F5528. LMP91000 VOUT is connected to Channel A0 of MSP430F5528.
//  Internal VREF (2.5V) is used as reference and repeated conversions are performed on Channel A0. 
//  Each conversion result is moved to an 8-element array called results[]. 
//  The results are averaged and converted to temperature (linear approximation)
//  using LMP91000 temperature sensor transfer table for temperature range of 20C to 50C.
//  Open locals window in debugger and view the results. Set a breakpoint in the  
//  measured_temp calculation line to see the array of conversion results in "results[]".
//  This can run even in LPM4 mode as ADC has its own clock
//
//                MSP430F5528
//             -----------------
//         /|\|              XIN|-
//          | |                 |  
//          --|RST          XOUT|-
//            |                 |
//            |     P3.0/UCB0SDA|<--> SDA 
//            |     P3.1/UCB0SCL|--> SCL 
//            |                 | 
//            |     P6.0/CB0/A0 |<-- P_VOUT 
//            |             P2.6|<-- MENB 
//            |                 | 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   July 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
// 
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.10
// Comments: Updated Description above
//           Cleaned up ISR mesured_temp calculation
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************

/*  Copyright 2007-2009 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller or used
  solely and exclusively in conjunction with a Texas Instruments radio frequency
  transceiver, which is integrated into your product. Other than for the
  foregoing purpose, you may not use, reproduce, copy, prepare derivative works
  of, modify, distribute, perform, display or sell this Software and/or its
  documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91000.h"
#include "TI_LMP91000_register_settings.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_i2c.h"
/*----------------------------------------------------------------------------*/
void ADC12_Init(void);                                                         // To init MSP430F5528 ADC12 & Start Conversion
void ADC10_Init(void);                                                         // To init MSP430F6736 ADC10 & Start Conversion
/*----------------------------------------------------------------------------*/
#define NUM_OF_RESULTS  8                                                      // Number of temp sensor samples to take
#define SCALE_FACTOR    3                                                      // For averaging converted samples
#define TEMP_EQN_OFFSET 1562.2                                                 // LMP91000 DS pg 12: V=(-8.16mV/C)*T+1562.2mV
#define TEMP_EQN_SLOPE  8.16                                                   // Linear approximation over temp range 20C to 50C
#define ADC12_RATIO     0.61035                                                // 2500/4096 (2.5V reference & 12bit converter)
#define ADC10_RATIO     1.46484375                                             // 1500/1024 (1.5V reference & 10bit converter)

#define ADC12_MODULE   2
#define ADC10_MODULE   1
#define ONBOARD_ADC_MODULE     ADC12_MODULE                                    // ADC module available in selected MSP430
                                                                               // choose between ADC12_MODULE & ADC10_MODULE
/*----------------------------------------------------------------------------*/
void main(void)
{

  uint8_t status = TI_LMP91000_NOT_READY;  

  WDTCTL = WDTPW + WDTHOLD;                                                    // Stop watchdog timer
  
  TI_LMP91000_LED_PxOUT |= TI_LMP91000_LED_PIN;                                // Set LED ON
  TI_LMP91000_LED_PxDIR |= TI_LMP91000_LED_PIN;                                // Set pin direction is output  

  I2CSetup(LMP91000_I2C_Address);                                              // Initialize I2C module
  
  TI_LMP91000_MENB_PxOUT &= ~TI_LMP91000_MENB_PIN;                             // Enable \MENB Pin
  TI_LMP91000_MENB_PxDIR |= TI_LMP91000_MENB_PIN;                              // Set pin direction is output
  
  while (status == TI_LMP91000_NOT_READY)
    status = LMP91000_I2CReadReg(TI_LMP91000_STATUS_REG);                      // Read device ready status
 
  LMP91000_I2CWriteReg(TI_LMP91000_MODECN_REG, TI_LMP91000_MODECN_REG_VALUE);  // Temperature Measurement Mode (TIA Off)

#if ONBOARD_ADC_MODULE == ADC12_MODULE
  ADC12_Init();                                                                // Initialize MSP430F5528 ADC12 & Start Conversion
#elif ONBOARD_ADC_MODULE == ADC10_MODULE
  ADC10_Init();                                                                // Initialize MSP430F6736 ADC10 & Start Conversion
#endif
     
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0, Enable interrupts
  __no_operation();                                                            // For debugger
}

/*----------------------------------------------------------------------------*/
#if ONBOARD_ADC_MODULE == ADC12_MODULE
/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the ADC12 Module (MSP430F5528) & Start Conversion
// Note: This function is for MSP430F5528 & might need modifications for other MSP430s
/*----------------------------------------------------------------------------*/
void ADC12_Init(void)
{
  TI_LMP91000_VOUT_ADC12_PxSEL |= TI_LMP91000_VOUT_ADC12_PIN;                  // Enable A/D channel A0
  REFCTL0 &= ~REFMSTR;                                                         // Reset REFMSTR to hand over control to 
                                                                               // ADC12_A ref control registers  
  ADC12CTL0 = ADC12ON+ADC12SHT0_8+ADC12MSC;                                    // Turn on ADC12, set sampling time
                                                                               // set multiple sample conversion
  ADC12CTL0 |= ADC12REFON+ADC12REF2_5V;                                        // Turn on Ref Gen & set to 2.5V
  ADC12MCTL0 = ADC12SREF_1;                                                    // Vr+ = Vref+ and Vr- = AVSS
  ADC12CTL1 = ADC12SHP+ADC12CONSEQ_2;                                          // Use sampling timer, set mode
  ADC12IE = ADC12IE0;                                                          // Enable ADC12IFG.0
  __delay_cycles(500);                                                         // delay to allow Ref to settle
  ADC12CTL0 |= ADC12ENC;                                                       // Enable conversions
  ADC12CTL0 |= ADC12SC;                                                        // Start conversion
  
}
/*----------------------------------------------------------------------------*/
// Description:
//   ADC12 Interrupt Service Routine (MSP430F5528)
// Note: This ISR is for MSP430F5528 & might need modifications for other MSP430s
/*----------------------------------------------------------------------------*/

#pragma vector=ADC12_VECTOR
__interrupt void ADC12ISR (void)
{
  static uint8_t index = 0;
  volatile static uint16_t results[NUM_OF_RESULTS];
  static uint32_t sum_adc_data = 0;
  volatile static float measured_temp, vout;

  switch(__even_in_range(ADC12IV,34))
  {
  case  0: break;                                                              // Vector  0:  No interrupt
  case  2: break;                                                              // Vector  2:  ADC overflow
  case  4: break;                                                              // Vector  4:  ADC timing overflow
  case  6:                                                                     // Vector  6:  ADC12IFG0
    results[index] = ADC12MEM0;                                                // Move results
    sum_adc_data += ADC12MEM0;
    index++;                                                                   // Increment results index, modulo; 
   
    if (index == NUM_OF_RESULTS)
    {
      sum_adc_data >>= SCALE_FACTOR;                                           // Divide by NUM_OF_RESULTS
      vout = sum_adc_data * ADC12_RATIO;                                       // LMP91000 vout
      measured_temp = (TEMP_EQN_OFFSET - vout) / TEMP_EQN_SLOPE;               
                                                                               // LMP91000 DS pg12: V=(-8.16mV/C)*T+1562.2mV
                                                                               // Linear approximation over temp range 20C to 50C
      sum_adc_data = 0;                                                        // Set Breakpoint here & see measured temperature
      index = 0;
    }
    break;
  case  8: break;                                                              // Vector  8:  ADC12IFG1
  case 10: break;                                                              // Vector 10:  ADC12IFG2
  case 12: break;                                                              // Vector 12:  ADC12IFG3
  case 14: break;                                                              // Vector 14:  ADC12IFG4
  case 16: break;                                                              // Vector 16:  ADC12IFG5
  case 18: break;                                                              // Vector 18:  ADC12IFG6
  case 20: break;                                                              // Vector 20:  ADC12IFG7
  case 22: break;                                                              // Vector 22:  ADC12IFG8
  case 24: break;                                                              // Vector 24:  ADC12IFG9
  case 26: break;                                                              // Vector 26:  ADC12IFG10
  case 28: break;                                                              // Vector 28:  ADC12IFG11
  case 30: break;                                                              // Vector 30:  ADC12IFG12
  case 32: break;                                                              // Vector 32:  ADC12IFG13
  case 34: break;                                                              // Vector 34:  ADC12IFG14
  default: break; 
  }
}
/*----------------------------------------------------------------------------*/
#elif ONBOARD_ADC_MODULE == ADC10_MODULE
/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the ADC10 Module (MSP430F6736) & Start Conversion
// Note: This function is for MSP430F6736 & might need modifications for other MSP430s
/*----------------------------------------------------------------------------*/
void ADC10_Init(void)
{
  TI_LMP91000_VOUT_ADC10_PxSEL |= TI_LMP91000_VOUT_ADC10_PIN;                  // Select pin as analog input A0  
  __disable_interrupt();                                                       // Disable interrupts; Port Map cnfg
  PMAPKEYID = PMAPKEY;                                                         // Enable access Port Mapping regs
  P1MAP2 = PM_ANALOG;                                                          // Enable A0
  PMAPKEYID = 0;                                                               // Disable access Port Mapping regs
  __enable_interrupt();                                                        // Re-enable all interrupts
  
  // By default, REFMSTR=1 => REFCTL is used to configure the internal reference
  while (REFCTL0 & REFGENBUSY) ;                                               // If ref generator busy, WAIT
  REFCTL0 |= REFVSEL_0 | REFON;                                                // Select internal ref = 1.5V
                                                                               // Internal Reference ON
  
  ADC10CTL0 = ADC10ON+ADC10SHT_2+ADC10MSC;                                     // Turn on ADC10, set sampling time
                                                                               // set multiple sample conversion
  ADC10CTL2 |= ADC10RES;                                                       // 10-bit conversion results  
  ADC10MCTL0 = ADC10SREF_1 | ADC10INCH_0;                                      // Vr+ = Vref+ and Vr- = AVSS
  ADC10CTL1 = ADC10SHP+ADC10CONSEQ_2;                                          // Use sampling timer, set mode
  ADC10IE = ADC10IE0;                                                          // Enable ADC10IFG.0
 
  __delay_cycles(100);                                                         // delay to allow Ref to settle 

  ADC10CTL0 |= ADC10ENC;                                                       // Enable conversions
  ADC10CTL0 |= ADC10SC;                                                        // Start conversion  
  
}
/*----------------------------------------------------------------------------*/
// Description:
//   ADC10 Interrupt Service Routine (MSP430F6736)
// Note: This ISR is for MSP430F6736 & might need modifications for other MSP430s
/*----------------------------------------------------------------------------*/

#pragma vector=ADC10_VECTOR
__interrupt void ADC10ISR (void)
{
  static uint8_t index = 0;
  volatile static uint16_t results[NUM_OF_RESULTS];
  static uint32_t sum_adc_data = 0;
  volatile static float measured_temp, vout;

  switch(__even_in_range(ADC10IV,34))
  {
  case  ADC10IV_NONE: break;                                                   // No interrupt
  case  ADC10IV_ADC10OVIFG: break;                                             // conversion result overflow
  case  ADC10IV_ADC10TOVIFG: break;                                            // conversion time overflow
  case  ADC10IV_ADC10HIIFG: break;                                             // ADC10HI
  case  ADC10IV_ADC10LOIFG: break;                                             // ADC10LO
  case  ADC10IV_ADC10INIFG: break;                                             // ADC10IN
  case  ADC10IV_ADC10IFG:                                                      // ADC10
    results[index] = ADC10MEM0;                                                // Move results
    sum_adc_data += ADC10MEM0;
    index++;                                                                   // Increment results index, modulo; 
   
    if (index == NUM_OF_RESULTS)
    {
      sum_adc_data >>= SCALE_FACTOR;                                           // Divide by NUM_OF_RESULTS
      vout = sum_adc_data * ADC12_RATIO;                                       // LMP91000 vout
      measured_temp = (TEMP_EQN_OFFSET - vout) / TEMP_EQN_SLOPE;               
                                                                               // LMP91000 DS pg12: V=(-8.16mV/C)*T+1562.2mV
                                                                               // Linear approximation over temp range 20C to 50C
      sum_adc_data = 0;                                                        // Set Breakpoint here & see measured temperature
      index = 0;
    }
    break;
  default: break; 
  }
}
/*----------------------------------------------------------------------------*/

#endif




