//******************************************************************************
//  Description: LMP91000 VOUT Measurement (No sensor: Vout = 67% of VREF = 1.67V)
//
//  Demo Application for MSP430/LMP91000 Interface Code Library v1.0
//  This is a new example using LMP91000EVM (not LMP91000SDEVAL) and the onboard ADC161S626. 
//  Code is applicable to both LMP91000 and LMP91002 devices.
//  LMP91000 VOUT is connected to ADC16S626 as shown in the LMP91000EVM User's Guide.
//  Each conversion result is moved to an 8-element array called results[].
//  The results are averaged and converted. Open a local window in debugger 
//  and view the results. Set a breakpoint in the line following vout calculation 
//  to see the array of conversion results & measured temperature.
//
//                MSP430F5528  
//             -----------------
//         /|\|              XIN|-
//          | |                 |  
//          --|RST          XOUT|-
//            |                 |
//            |     P3.0/UCB0SDA|<--> SDA 
//            |     P3.1/UCB0SCL|--> SCL 
//            |                 | 
//            |             P2.6|<-- MENB 
//            |                 | 
//            |    P4.5/UCA1SOMI|<-- ADC16S626_SDO 
//            |     P4.0/UCA1CLK|--> ADC16S626_CLK 
//            |             P2.4|--> ADC16S626_CSB 
//            |                 | 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   July 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************
/*  Copyright 2011-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is 
  integrated into your product. Other than for the foregoing purpose, you may 
  not use, reproduce, copy, prepare derivative works of, modify, distribute, 
  perform, display or sell this Software and/or its documentation for any 
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91000.h"
#include "TI_LMP91000_register_settings.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_i2c.h"

void timerA0_init(void);                                                       // initialize timerA0 to generate interrupt every sec
                                                                               // for temperature measurement
#define NUM_OF_RESULTS      8                                                  // Number of temp sensor samples to take
#define SCALE_FACTOR        3                                                  // For averaging converted samples
#define ADC16_RATIO         0.07630                                            // 2500/3268(2500mV ref & 16bit converter, 2s comp o/p)
#define INTERVAL_1S         16666                                              // 1sec timer interval
#define AIN_MINUS       2501                                                   // AIN_MINUS = VREF = 2501mV

//******************************************************************************
void main(void)
{

  uint8_t status = TI_LMP91000_NOT_READY;
  uint8_t read_val[2];                                                         // buffer to store register values
  
  WDTCTL = WDTPW + WDTHOLD;                                                    // Stop watchdog timer
  
  TI_LMP91000_LED_PxOUT |= TI_LMP91000_LED_PIN;                                // Set LED ON
  TI_LMP91000_LED_PxDIR |= TI_LMP91000_LED_PIN;                                // Set pin direction is output    
  
  I2CSetup(LMP91000_I2C_Address);                                              // Initialize I2C module
  
  TI_LMP91000_MENB_PxOUT &= ~TI_LMP91000_MENB_PIN;                             // Enable \MENB Pin
  TI_LMP91000_MENB_PxDIR |= TI_LMP91000_MENB_PIN;                              // Set pin direction is output
  
  while (status == TI_LMP91000_NOT_READY)
    status = LMP91000_I2CReadReg(TI_LMP91000_STATUS_REG);                      // Read device ready status
 
  LMP91000_I2CWriteReg(TI_LMP91000_LOCK_REG, TI_LMP91000_WRITE_UNLOCK);        // unlock the registers for write
 
  LMP91000_I2CWriteReg(TI_LMP91000_TIACN_REG, TI_LMP91000_TIACN_REG_VALUE);    // Modify TIA control register 
  LMP91000_I2CWriteReg(TI_LMP91000_REFCN_REG, TI_LMP91000_REFCN_REG_VALUE);    // Modify REF control register
  
  read_val[0]   = LMP91000_I2CReadReg(TI_LMP91000_TIACN_REG);                  // Read to confirm register is modified
  read_val[1]   = LMP91000_I2CReadReg(TI_LMP91000_REFCN_REG);                  // Read to confirm register is modified  
  
  if ((read_val[0] != TI_LMP91000_TIACN_REG_VALUE) || 
      (read_val[1] != TI_LMP91000_REFCN_REG_VALUE))                            // test values took effect  
    while (1);                                                                 // otherwise error
  
  LMP91000_I2CWriteReg(TI_LMP91000_LOCK_REG, TI_LMP91000_WRITE_LOCK);          // lock the registers 
  LMP91000_I2CWriteReg(TI_LMP91000_MODECN_REG, TI_LMP91000_MODECN_REG_VALUE);  // 3-lead amperometric cell
  
  TI_ADC16S626_SPISetup();                                                     // setup onboard ADS14S626 SPI interface
  timerA0_init();                                                              // initialize timerA0 for interrupt every 1sec                  
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0, enable interrupts
  __no_operation();                                                            // For debugger 

} 
//------------------------------------------------------------------------------
//  void timerA0_init(void)
//
//  DESCRIPTION:
//  Initialize timer_A0 to generat 1sec interrupt.   
//  Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
void timerA0_init(void)
{
        
  TA0CCTL0 = CCIE;                                                             // TA2CCR0 interrupt enabled
  TA0CCR0 = INTERVAL_1S;                                                       // Initialize for 1sec interval
  TA0EX0 = TAIDEX_7;                                                           // Divide clock (selected below) further by 8	
  TA0CTL = TASSEL_2 + MC_1 + TACLR + ID_3;                                     // SMCLK, upmode, clear TAR, divide by 8
    
}
//------------------------------------------------------------------------------
// Timer A0 interrupt service routine
// Note: This ISR is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR (void)
{
  uint8_t i;
  static uint16_t results[NUM_OF_RESULTS];                                     // To store ADC16 output
  static uint32_t sum_adc_data = 0;                                            // accumulate and avg adc results
  volatile static float vout;                                                  // lmp91000 vout 
  
  for(i=0; i<8;i++)
  {
    results[i] = TI_ADC16S626_SPIReadReg();                                    // read and store ADC16 output
    results[i] = results[i] ^ 0x8000;                                          // make adc output 16bit unsigned    
    sum_adc_data += results[i];                                                // accumulate result
    __delay_cycles(4);                                                         // acquisition time                             
  }
  sum_adc_data >>= SCALE_FACTOR;                                               // Average the accumulated sum
  vout = sum_adc_data * ADC16_RATIO;                                           // LMP91000 vout  
  sum_adc_data = 0;                                                            // Set Breakpoint here & see measured vout     
  
}  
//------------------------------------------------------------------------------
