//******************************************************************************
//  Description:  This file contains functions that allow the MSP430 device to
//  access the I2C interface of the LMP91000.  There are multiple
//  instances of each function; the one to be compiled is selected by the
//  system variable TI_LMP91000_SER_INTF, defined in "TI_LMP91000_hardware_board.h".
//
//  MSP430/LMP91000 Interface Code Library v1.0
// 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   October 2011
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************
/*  Copyright 2007-2009 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller or used
  solely and exclusively in conjunction with a Texas Instruments radio frequency
  transceiver, which is integrated into your product. Other than for the
  foregoing purpose, you may not use, reproduce, copy, prepare derivative works
  of, modify, distribute, perform, display or sell this Software and/or its
  documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/


#include <stdint.h>
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_i2c.h"

//******************************************************************************
// Support for 1xx USART0
//******************************************************************************
#if TI_LMP91000_SER_INTF == TI_LMP91000_SER_INTF_USART0_1xx

int8_t PtrTransmit;
uint8_t I2CBufferArray[16];
uint8_t I2CBuffer;

/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the I2C Module
/*----------------------------------------------------------------------------*/
void I2CSetup(uint8_t device_i2c_address)
{
  TI_LMP91000_I2C_USART0_PxSEL |= TI_LMP91000_I2C_USART0_SDA + 
                                  TI_LMP91000_I2C_USART0_SCL;                   // Assign I2C pins to USART0
  TI_LMP91000_I2C_USART0_PxDIR &= ~(TI_LMP91000_I2C_USART0_SDA + TI_LMP91000_I2C_USART0_SCL);  

  // Recommended initialisation steps of I2C module as shown in User Guide:
  U0CTL |= I2C+SYNC;                                                           // (1) Select I2C mode with SWRST=1
  U0CTL &= ~I2CEN;                                                             // (2) disable the I2C module
                                                                               // (3) Configure the I2C module with I2CEN=0 :
                                                                               // U0CTL default settings:
                                                                               // 7-bit addressing, no DMA, no feedback
  I2CTCTL = I2CTRX+I2CSSEL_2;                                                  // byte mode, repeat mode, clock source = SMCLK,
                                                                               // transmit mode
  I2CSA = device_i2c_address;                                                  // Slave Address - defines the
                                                                               // control byte that is sent to the
                                                                               // EEPROM.
  I2COA = 0x01A5;                                                              // own address.
  I2CPSC = 0x00;                                                               // I2C clock = clock source/1
  I2CSCLH = 0x03;                                                              // SCL high period = 5*I2C clock
  I2CSCLL = 0x03;                                                              // SCL low period  = 5*I2C clock
  U0CTL |= I2CEN;                                                              // (4) set I2CEN via software

  if (I2CDCTL&I2CBUSY)                                                         // test if bus to be free
  {                                                                            // otherwise a manual Clock on is
                                                                               // generated
    TI_LMP91000_I2C_USART0_PxSEL &= ~TI_LMP91000_I2C_USART0_SCL;               // Select Port function for SCL
    TI_LMP91000_I2C_USART0_PxOUT &= ~TI_LMP91000_I2C_USART0_SCL;               //
    TI_LMP91000_I2C_USART0_PxDIR |= TI_LMP91000_I2C_USART0_SCL;                // drive SCL low
    TI_LMP91000_I2C_USART0_PxSEL |= TI_LMP91000_I2C_USART0_SDA +
                                    TI_LMP91000_I2C_USART0_SCL;                // select module function for the
                                                                               // used I2C pins
  } 
}

/*---------------------------------------------------------------------------*/
// Description:
//   Initialization of the I2C Module for Write operation.
/*---------------------------------------------------------------------------*/
void I2CWriteInit(void)
{ 
  U0CTL |= MST;                                                                // define Master Mode
  I2CTCTL |= I2CTRX;                                                           // I2CTRX=1 => Transmit Mode (R/W bit = 0)
  I2CIFG &= ~TXRDYIFG;
  I2CIE = TXRDYIE;                                                             // enable Transmit ready interrupt  
}

/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the I2C Module for Read operation.
/*----------------------------------------------------------------------------*/
void I2CReadInit(void)
{
  I2CTCTL &= ~I2CTRX;                                                          // I2CTRX=0 => Receive Mode
                                                                               //   (R/W bit = 1)
  U0CTL |= MST;                                                                // define Master Mode
  I2CIFG &= ~RXRDYIFG;
  I2CIE &= ~TXRDYIE;                                                           // disable Transmit ready interrupt
  I2CIE |= RXRDYIE;                                                            // enable Receive ready interrupt
}

/*----------------------------------------------------------------------------*/
// Description:
//   Register Write Operation. The communication via the I2C bus with a LMP91000
//   is realized. A data byte is written into a user defined address.
/*----------------------------------------------------------------------------*/
void LMP91000_I2CWriteReg(uint8_t address, uint8_t data)
{

  while (I2CDCTL&I2CBUSY);                                                     // wait until I2C module has
                                                                               // finished all operations

  I2CBufferArray[1] = address;                                                 // Byte address.
  I2CBufferArray[0] = data;
  PtrTransmit = 1;                                                             // set I2CBufferArray Pointer

  I2CWriteInit();
  I2CNDAT = 2;                                                                 // 1 control byte + 2 bytes should
                                                                               //   be transmitted
  I2CTCTL |= I2CSTT;                                                           // start condition generation
                                                                               // => I2C communication is started
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0 w/ interrupts
  I2CTCTL |= I2CSTP;                                                           // I2C stop condition
  while(I2CTCTL & I2CSTP);                                                     // Ensure stop condition got sent
  
}
/*----------------------------------------------------------------------------*/
// Description:
//   Register Read Operation. Data byte is read from the LMP91000. The LMP91000
//   address is defined with the parameter address.
/*----------------------------------------------------------------------------*/
uint8_t LMP91000_I2CReadReg(uint8_t address)
{

  while (I2CDCTL & I2CBUSY);                                                   // wait until I2C module has
                                                                               // finished all operations

  I2CBufferArray[0] = address;                                                 // store single bytes that have to
                                                                               // be sent in the I2CBuffer.
  PtrTransmit = 0;                                                             // set I2CBufferArray Pointer

  // Write Address first
  I2CWriteInit();
  I2CNDAT = 1;                                                                 // 1 control byte + 1 byte should
                                                                               //   be transmitted
  I2CTCTL |= I2CSTT;                                                           // start condition generation
                                                                               // => I2C communication is started
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0 w/ interrupts

  // Read Data byte
  I2CReadInit();

  I2CNDAT = 1;                                                                 // 1 byte should be received
  I2CTCTL |= I2CSTT;                                                           // start receiving
                                                                               // re-start condition
  while(I2CTCTL & I2CSTT);                                                     // Start condition sent?
  I2CTCTL |= I2CSTP;                                                           // I2C stop condition
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0 w/ interrupts
  while(I2CTCTL & I2CSTP);                                                     // Ensure stop condition got sent
  return I2CBuffer;  
}

/*---------------------------------------------------------------------------*/
/*  Interrupt Service Routines                                               */
/*     Note that the Compiler version is checked in the following code and   */
/*     depending of the Compiler Version the correct Interrupt Service       */
/*     Routine definition is used.                                           */
/*---------------------------------------------------------------------------*/
#if __VER__ < 200
    interrupt [USART0TX_VECTOR] void ISR_I2C(void)
#else
    #pragma vector=USART0TX_VECTOR
    __interrupt void ISR_I2C(void)
#endif
{
  switch (__even_in_range(I2CIV, I2CIV_STT))
  {
    case I2CIV_AL:                                                             /* I2C interrupt vector: Arbitration lost (ALIFG) */
      break;
    case I2CIV_NACK:                                                           /* I2C interrupt vector: No acknowledge (NACKIFG) */
      break;
    case I2CIV_OA:                                                             /* I2C interrupt vector: Own address (OAIFG) */
      break;
    case I2CIV_ARDY:                                                           /* I2C interrupt vector: Access ready (ARDYIFG) */
      break;
    case I2CIV_RXRDY:                                                          /* I2C interrupt vector: Receive ready (RXRDYIFG) */
      I2CBuffer = I2CDRB;                                                      // store received data in buffer
      __bic_SR_register_on_exit(LPM0_bits);                                    // Exit LPM0
      break;
    case I2CIV_TXRDY:                                                          /* I2C interrupt vector: Transmit ready (TXRDYIFG) */
      while(!(I2CDCTL & I2CTXUDF));                                            // Checks if TX buff is empty
      I2CDRB = I2CBufferArray[PtrTransmit];
      PtrTransmit--;
      if (PtrTransmit < 0)
      {
        I2CIE &= ~TXRDYIE;                                                     // disable interrupts
        I2CIFG &= ~TXRDYIFG;                                                   // Clear USCI_B0 TX int flag
        __bic_SR_register_on_exit(LPM0_bits);                                  // Exit LPM0
      }
      break;
    case I2CIV_GC:                                                             /* I2C interrupt vector: General call (GCIFG) */
      break;
    case I2CIV_STT:                                                            /* I2C interrupt vector: Start condition (STTIFG) */
      break;
  }
}

#endif
